@@ -24,6 +24,7 @@
 #define __GINAC_FUNCTION_H__
 
 #include "exprseq.h"
+#include "infoflagbase.h"
 
 #include <string>
 #include <vector>
@@ -52,6 +53,13 @@ const unsigned NAME##_NPARAMS = 3; \
 template<typename T1, typename T2, typename T3> const GiNaC::function NAME(const T1 & p1, const T2 & p2, const T3 & p3) { \
 	return GiNaC::function(NAME##_SERIAL::serial, GiNaC::ex(p1), GiNaC::ex(p2), GiNaC::ex(p3)); \
 }
+
+#define DECLARE_FUNCTION_6P(NAME) \
+class NAME##_SERIAL { public: static unsigned serial; }; \
+const unsigned NAME##_NPARAMS = 6; \
+template<typename T1, typename T2, typename T3, typename T4, typename T5, typename T6> const GiNaC::function NAME(const T1 & p1, const T2 & p2, const T3 & p3, const T4 & p4, const T5 & p5, const T6 & p6) { \
+	return GiNaC::function(NAME##_SERIAL::serial, GiNaC::ex(p1), GiNaC::ex(p2), GiNaC::ex(p3), GiNaC::ex(p4), GiNaC::ex(p5), GiNaC::ex(p6)); \
+}
 // end of generated lines
 
 #define REGISTER_FUNCTION(NAME,OPT) \
@@ -69,6 +77,7 @@ typedef ex (* conjugate_funcp)();
 typedef ex (* real_part_funcp)();
 typedef ex (* imag_part_funcp)();
 typedef ex (* derivative_funcp)();
+typedef ex (* expl_derivative_funcp)();
 typedef ex (* power_funcp)();
 typedef ex (* series_funcp)();
 typedef ex (* subs_funcp)(PyObject* parent);
@@ -78,9 +87,11 @@ typedef void (* print_funcp)();
 typedef ex (* eval_funcp_1)(const ex &);
 typedef ex (* eval_funcp_2)(const ex &, const ex &);
 typedef ex (* eval_funcp_3)(const ex &, const ex &, const ex &);
+typedef ex (* eval_funcp_6)(const ex &, const ex &, const ex &, const ex &, const ex &, const ex &);
 typedef ex (* evalf_funcp_1)(const ex &, PyObject* parent);
 typedef ex (* evalf_funcp_2)(const ex &, const ex &, PyObject* parent);
 typedef ex (* evalf_funcp_3)(const ex &, const ex &, const ex &, PyObject* parent);
+typedef ex (* evalf_funcp_6)(const ex &, const ex &, const ex &, const ex &, const ex &, const ex &, PyObject* parent);
 
 typedef ex (* conjugate_funcp_1)(const ex &);
 typedef ex (* conjugate_funcp_2)(const ex &, const ex &);
@@ -96,6 +107,10 @@ typedef ex (* imag_part_funcp_3)(const ex &, const ex &, const ex &);
 typedef ex (* derivative_funcp_1)(const ex &, unsigned);
 typedef ex (* derivative_funcp_2)(const ex &, const ex &, unsigned);
 typedef ex (* derivative_funcp_3)(const ex &, const ex &, const ex &, unsigned);
+typedef ex (* derivative_funcp_6)(const ex &, const ex &, const ex &, const ex &, const ex &, const ex &, unsigned);
+typedef ex (* expl_derivative_funcp_1)(const ex &, const symbol &);
+typedef ex (* expl_derivative_funcp_2)(const ex &, const ex &, const symbol &);
+typedef ex (* expl_derivative_funcp_3)(const ex &, const ex &, const ex &, const symbol &);
 typedef ex (* power_funcp_1)(const ex &, const ex &);
 typedef ex (* power_funcp_2)(const ex &, const ex &, const ex &);
 typedef ex (* power_funcp_3)(const ex &, const ex &, const ex &, const ex &);
@@ -116,6 +131,7 @@ typedef ex (* conjugate_funcp_exvector)(const exvector &);
 typedef ex (* real_part_funcp_exvector)(const exvector &);
 typedef ex (* imag_part_funcp_exvector)(const exvector &);
 typedef ex (* derivative_funcp_exvector)(const exvector &, unsigned);
+typedef ex (* expl_derivative_funcp_exvector)(const exvector &, const symbol &);
 typedef ex (* power_funcp_exvector)(const exvector &, const ex &);
 typedef ex (* series_funcp_exvector)(const exvector &, const relational &, int, unsigned);
 typedef void (* print_funcp_exvector)(const exvector &, const print_context &);
@@ -142,10 +158,12 @@ public:
     function_options & eval_func(eval_funcp_1 e);
     function_options & eval_func(eval_funcp_2 e);
     function_options & eval_func(eval_funcp_3 e);
+    function_options & eval_func(eval_funcp_6 e);
 
     function_options & evalf_func(evalf_funcp_1 ef);
     function_options & evalf_func(evalf_funcp_2 ef);
     function_options & evalf_func(evalf_funcp_3 ef);
+    function_options & evalf_func(evalf_funcp_6 ef);
 
     function_options & conjugate_func(conjugate_funcp_1 d);
     function_options & conjugate_func(conjugate_funcp_2 d);
@@ -162,6 +180,11 @@ public:
     function_options & derivative_func(derivative_funcp_1 d);
     function_options & derivative_func(derivative_funcp_2 d);
     function_options & derivative_func(derivative_funcp_3 d);
+    function_options & derivative_func(derivative_funcp_6 d);
+
+    function_options & expl_derivative_func(expl_derivative_funcp_1 d);
+    function_options & expl_derivative_func(expl_derivative_funcp_2 d);
+    function_options & expl_derivative_func(expl_derivative_funcp_3 d);
 
     function_options & power_func(power_funcp_1 d);
     function_options & power_func(power_funcp_2 d);
@@ -219,7 +242,7 @@ public:
 	function_options & series_func(PyObject* e);
 	function_options & subs_func(PyObject* e);
 
-	function_options & set_return_type(unsigned rt, tinfo_t rtt=NULL);
+	function_options & set_return_type(unsigned rt, tinfo_t rtt=nullptr);
 	function_options & do_not_evalf_params();
 	function_options & do_not_apply_chain_rule();
 	function_options & remember(unsigned size, unsigned assoc_size=0,
@@ -247,9 +270,9 @@ public:
 		subs_python_f           = 0x0100,
 	};
 
-protected:
-	bool has_derivative() const { return derivative_f != NULL; }
-	bool has_power() const { return power_f != NULL; }
+public:
+	bool has_derivative() const { return derivative_f != nullptr; }
+	bool has_power() const { return power_f != nullptr; }
 	void test_and_set_nparams(unsigned n);
 	void set_print_func(unsigned id, print_funcp f);
 
@@ -264,6 +287,7 @@ protected:
 	real_part_funcp real_part_f;
 	imag_part_funcp imag_part_f;
 	derivative_funcp derivative_f;
+        expl_derivative_funcp expl_derivative_f;
 	power_funcp power_f;
 	series_funcp series_f;
         subs_funcp subs_f;
@@ -287,6 +311,7 @@ protected:
 	bool real_part_use_exvector_args;
 	bool imag_part_use_exvector_args;
 	bool derivative_use_exvector_args;
+        bool expl_derivative_use_exvector_args;
 	bool power_use_exvector_args;
 	bool series_use_exvector_args;
 	bool print_use_exvector_args;
@@ -310,7 +335,7 @@ class function : public exprseq
 {
 	GINAC_DECLARE_REGISTERED_CLASS(function, exprseq)
 
-	friend struct print_order;
+	friend class print_order;
 	// CINT has a linking problem
 #ifndef __MAKECINT__
 	friend void ginsh_get_ginac_functions();
@@ -325,38 +350,43 @@ class function : public exprseq
 	// other constructors
 public:
 	function(unsigned ser);
-	// the following lines have been generated for max. 14 parameters
+	// the following lines have been generated for some parameters
     function(unsigned ser, const ex & param1);
     function(unsigned ser, const ex & param1, const ex & param2);
     function(unsigned ser, const ex & param1, const ex & param2, const ex & param3);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5, const ex & param6);
 	// end of generated lines
-	function(unsigned ser, const exprseq & es);
+	function(unsigned ser, exprseq  es);
 	function(unsigned ser, const exvector & v, bool discardable = false);
-	function(unsigned ser, std::auto_ptr<exvector> vp);
+	function(unsigned ser, std::unique_ptr<exvector> vp);
 
 	// functions overriding virtual functions from base classes
 public:
-	void print(const print_context & c, unsigned level = 0) const;
-	unsigned precedence() const {return 70;}
-	ex expand(unsigned options=0) const;
-	ex eval(int level=0) const;
-	ex evalf(int level=0, PyObject* parent=NULL) const;
-	unsigned calchash() const;
-	ex series(const relational & r, int order, unsigned options = 0) const;
-        ex subs(const exmap & m, unsigned options = 0) const;
-	ex thiscontainer(const exvector & v) const;
-	ex thiscontainer(std::auto_ptr<exvector> vp) const;
-	ex conjugate() const;
-	ex real_part() const;
-	ex imag_part() const;
-	bool info(unsigned inf) const;
+	void print(const print_context & c, unsigned level = 0) const override;
+	unsigned precedence() const override {return 70;}
+	ex expand(unsigned options=0) const override;
+	ex eval(int level=0) const override;
+	ex evalf(int level=0, PyObject* parent=nullptr) const override;
+	long calchash() const override;
+	ex series(const relational & r, int order, unsigned options = 0) const override;
+        void useries(flint_series_t& fp, int order) const override;
+        ex subs(const exmap & m, unsigned options = 0) const override;
+	ex normal(exmap & repl, exmap & rev_lookup, int level = 0, unsigned options = 0) const override;
+	ex thiscontainer(const exvector & v) const override;
+	ex thiscontainer(std::unique_ptr<exvector> vp) const override;
+	ex conjugate() const override;
+	ex real_part() const override;
+	ex imag_part() const override;
+	bool info(unsigned inf) const override;
 	//int compare(const basic &other) const;
 protected:
-	ex derivative(const symbol & s) const;
-	bool is_equal_same_type(const basic & other) const;
-	bool match_same_type(const basic & other) const;
-	unsigned return_type() const;
-	tinfo_t return_type_tinfo() const;
+	ex derivative(const symbol & s) const override;
+	bool is_equal_same_type(const basic & other) const override;
+	bool match_same_type(const basic & other) const override;
+	unsigned return_type() const override;
+	tinfo_t return_type_tinfo() const override;
 	
 	// new virtual functions which can be overridden by derived classes
 	// none
@@ -364,6 +394,7 @@ protected:
 	// non-virtual functions in this class
 protected:
 	ex pderivative(unsigned diff_param) const; // partial differentiation
+        ex expl_derivative(const symbol & s) const; // partial differentiation
 	bool lookup_remember_table(ex & result) const;
 	void store_remember_table(ex const & result) const;
 public:
@@ -374,11 +405,16 @@ public:
 	static unsigned find_function(const std::string &name, unsigned nparams);
 	unsigned get_serial() const {return serial;}
 	std::string get_name() const;
+	unsigned get_domain() const { return domain; }
+	void set_domain(unsigned d);
+	void set_info(unsigned flag, bool value=true) { iflags.set(flag, value); }
 
 // member variables
 
 protected:
 	unsigned serial;
+	unsigned domain;
+	infoflagbase iflags;
 };
 
 // utility functions/macros
@@ -393,6 +429,14 @@ inline bool is_the_function(const ex & x)
 // Check whether OBJ is the specified symbolic function.
 #define is_ex_the_function(OBJ, FUNCNAME) (GiNaC::is_the_function<FUNCNAME##_SERIAL>(OBJ))
 
+bool has_function(const ex & x);
+bool has_function(const ex& x, const std::string& s);
+bool has_function(const std::vector<std::string>& v);
+bool has_function(const ex& x,
+                const std::vector<std::string>& v,
+                bool all);
+bool has_symbol_or_function(const ex & x);
+
 } // namespace GiNaC
 
 #endif // ndef __GINAC_FUNCTION_H__
