@@ -22,6 +22,7 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "infinity.h"
 #include "symbol.h"
@@ -1106,10 +1107,22 @@ REGISTER_FUNCTION(csc, eval_func(csc_eval).
 // inverse sine (arc sine)
 //////////
 
+// Needed because there is no RR member
+static ex asin_evalf(const ex & x, PyObject* parent)
+{
+	if (is_exactly_a<numeric>(x))
+		return asin(ex_to<numeric>(x), parent);
+
+	return asin(x).hold();
+}
+
 static ex asin_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // asin() is odd
+        if (x.info(info_flags::negative))
+                return -asin(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// asin(0) -> 0
 		if (x.is_zero())
 			return x;
@@ -1119,17 +1132,9 @@ static ex asin_eval(const ex & x)
 			return numeric(1,6)*Pi;
 
 		// asin(1) -> Pi/2
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_2*Pi;
 
-		// asin(-1/2) -> -Pi/6
-		if (x.is_equal(_ex_1_2))
-			return numeric(-1,6)*Pi;
-
-		// asin(-1) -> -Pi/2
-		if (x.is_equal(_ex_1))
-			return _ex_1_2*Pi;
-
 		if (x.info(info_flags::inexact)) {
                         const numeric& num = ex_to<numeric>(x);
                         if (num.is_real()
@@ -1140,9 +1145,6 @@ static ex asin_eval(const ex & x)
 	        		return asin(ex_to<numeric>(x));
                 }
 
-		// asin() is odd
-		if (x.info(info_flags::negative))
-			return -asin(-x);
 	}
 	
 	// asin(oo) -> error
@@ -1153,6 +1155,12 @@ static ex asin_eval(const ex & x)
 		throw (std::runtime_error("arcsin_eval(): arcsin(infinity) encountered"));
 	}
 
+        if (x.is_equal(mul(pow(_ex2, _ex1_2), _ex1_2)))
+                return mul(Pi, _ex1_4);
+
+        if (x.is_equal(mul(pow(_ex3, _ex1_2), _ex1_2)))
+                return mul(Pi, _ex1_3);
+
 	return asin(x).hold();
 }
 
@@ -1176,6 +1184,7 @@ static ex asin_conjugate(const ex & x)
 }
 
 REGISTER_FUNCTION(asin, eval_func(asin_eval).
+                        evalf_func(asin_evalf).
                         derivative_func(asin_deriv).
                         conjugate_func(asin_conjugate).
 			set_name("arcsin", "\\arcsin"));
@@ -1189,7 +1198,7 @@ static ex acos_eval(const ex & x)
 	if (is_exactly_a<numeric>(x)) {
 
 		// acos(1) -> 0
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex0;
 
 		// acos(1/2) -> Pi/3
@@ -1205,7 +1214,7 @@ static ex acos_eval(const ex & x)
 			return numeric(2,3)*Pi;
 
 		// acos(-1) -> Pi
-		if (x.is_equal(_ex_1))
+		if (x.is_minus_one())
 			return Pi;
 
 		if (x.info(info_flags::inexact)) {
@@ -1264,30 +1273,25 @@ REGISTER_FUNCTION(acos, eval_func(acos_eval).
 
 static ex atan_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // atan() is odd
+        if (x.info(info_flags::negative))
+                return -atan(-x);
 
+        if (is_exactly_a<numeric>(x)) {
 		// atan(0) -> 0
 		if (x.is_zero())
 			return _ex0;
 
 		// atan(1) -> Pi/4
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_4*Pi;
 
-		// atan(-1) -> -Pi/4
-		if (x.is_equal(_ex_1))
-			return _ex_1_4*Pi;
-
-		if (x.is_equal(I) || x.is_equal(-I))
+		if (x.is_equal(I))
 			throw (pole_error("atan_eval(): logarithmic pole",0));
 
 		// atan(float) -> float
 		if (x.info(info_flags::inexact))
 			return atan(ex_to<numeric>(x));
-
-		// atan() is odd
-		if (x.info(info_flags::negative))
-			return -atan(-x);
 	}
 	
 	// arctan(oo) -> Pi/2
@@ -1302,7 +1306,13 @@ static ex atan_eval(const ex & x)
 		// x is UnsignedInfinity
 		throw (std::runtime_error("arctan_eval(): arctan(unsigned_infinity) encountered"));
 	}
-		
+
+        if (x.is_equal(pow(_ex3, _ex1_2)))
+                return mul(Pi, _ex1_3);
+
+        if (x.is_equal(mul(pow(_ex3, _ex1_2), _ex1_3)))
+                return mul(Pi, numeric(1,6));
+
 	return atan(x).hold();
 }
 
@@ -1516,10 +1526,10 @@ static ex acot_eval(const ex & x)
 		if (x.is_zero())
 			return _ex1_2*Pi;
 
-		if (x.is_equal(_ex1))
+		if (x.is_one())
 			return _ex1_4*Pi;
 
-		if (x.is_equal(_ex_1))
+		if (x.is_minus_one())
 			return _ex_1_4*Pi;
 
 		if (x.is_equal(I) || x.is_equal(-I))
@@ -1573,7 +1583,13 @@ REGISTER_FUNCTION(acot, eval_func(acot_eval).
 static ex asec_evalf(const ex & x, PyObject* parent)
 {
 	if (is_exactly_a<numeric>(x))
-		return acos(ex_to<numeric>(x).inverse());
+        {
+                numeric num = ex_to<numeric>(x);
+                if (num.is_real()
+                    and (num > *_num_1_p and num < *_num1_p))
+                        return NaN;
+                return acos(num.inverse());
+        }
 
 	return asec(x).hold();
 }
@@ -1589,7 +1605,7 @@ static ex asec_eval(const ex & x)
                 if (num.is_equal(*_num_1_p))
                         return Pi;
                 if (num.info(info_flags::inexact))
-                        return acos(num.inverse());
+                        return asec_evalf(x, nullptr);
 	}
 
 	if (x.info(info_flags::infinity)) {
@@ -1637,8 +1653,13 @@ REGISTER_FUNCTION(asec, eval_func(asec_eval).
 // Needed because there is no Python RR equivalent
 static ex acsc_evalf(const ex & x, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(x))
-		return asin(ex_to<numeric>(x).inverse());
+	if (is_exactly_a<numeric>(x)) {
+                const numeric& num = ex_to<numeric>(x);
+                if (num.is_real()
+                    and (num > *_num_1_p and num < *_num1_p))
+                        return NaN;
+		return asin(num.inverse());
+        }
 
 	return acsc(x).hold();
 }
