@@ -44,27 +44,20 @@
 #include "inifcns.h"
 #include "function.h"
 #include "utils.h"
+#include "wildcard.h"
 
 namespace GiNaC {
 
-void Log(const power_ocvector_map& m)
+void Log(const power_ocvector_map& m, std::string str)
 {
+        if (not str.empty())
+                std::cerr << str << ":\n";
         for (auto item : m) {
                 std::cerr << item.first << ":\n";
                 Log(item.second);
         }
 }
 
-static CanonicalForm num2canonical(const numeric& n)
-{
-        if (n.is_long() or n.is_mpz() or n.is_mpq())
-                return n.to_canonical();
-
-        std::cerr << "\nn = " << n << "\n";
-        std::cerr << "is_pyobject = " << n.is_pyobject() << "\n";
-        throw std::runtime_error("num2canonical: can't happen");
-}
-
 static CanonicalForm replace_with_symbol(const ex& e, ex_int_map& map, exvector& revmap)
 {
         // Expression already replaced? Then return the assigned symbol
@@ -80,6 +73,45 @@ static CanonicalForm replace_with_symbol(const ex& e, ex_int_map& map, exvector&
         return Variable(index);
 }
 
+static CanonicalForm num2canonical(const numeric& n, ex_int_map& map, exvector& revmap)
+{
+        try {
+                return n.to_canonical();
+        }
+        catch (std::runtime_error) {
+                if (not n.is_real()) {
+                        numeric re = n.real();
+                        numeric im = n.imag();
+                        CanonicalForm re_p, im_p;
+                        if (re.is_rational())
+                                re_p = re.to_canonical();
+                        else {
+                                if (re.is_positive())
+                                        re_p = replace_with_symbol(re,
+                                                        map, revmap);
+                                else
+                                        re_p = -replace_with_symbol(-re,
+                                                        map, revmap);
+                        }
+                        if (im.is_rational())
+                                im_p = im.to_canonical();
+                        else {
+                                if (im.is_positive())
+                                        im_p = replace_with_symbol(im,
+                                                        map, revmap);
+                                else
+                                        im_p = -replace_with_symbol(-im,
+                                                        map, revmap);
+                        }
+                        return re_p + im_p * replace_with_symbol(I, map, revmap);
+                }
+                if (n.is_positive())
+                        return replace_with_symbol(n, map, revmap);
+                else
+                        return -replace_with_symbol(n.negative(), map, revmap);
+        }
+}
+
 static symbol symbol_E;
 
 static void add_to_pomap(power_ocvector_map& pomap,
@@ -103,27 +135,21 @@ void ex::collect_powers(power_ocvector_map& pomap) const
                 if (is_exactly_a<numeric>(the_pow.op(1))) {
                         numeric n = ex_to<numeric>(the_pow.op(1));
                         if (n.is_rational()) {
-                                if (n.is_negative())
-                                        add_to_pomap(pomap, the_pow.op(0), _ex_1, -n);
-                                else
                                         add_to_pomap(pomap, the_pow.op(0), _ex1, n);
                         }
                 }
-                if (is_exactly_a<mul>(the_pow.op(1))) {
-                        mul m = ex_to<mul>(the_pow.op(1));
-                        numeric oc = m.overall_coeff;
-                        if (oc.is_rational()) {
-                                if (oc.is_negative()) {
-                                        m.overall_coeff = *_num_1_p;
-                                        ex mm = m.eval();
-                                        add_to_pomap(pomap, the_pow.op(0), mm, -oc);
-                                }
-                                else {
+                else {
+                        numeric oc = *_num1_p;
+                        ex e = the_pow.op(1);
+                        if (is_exactly_a<mul>(e)) {
+                                mul m = ex_to<mul>(e);
+                                oc = m.overall_coeff;
+                                if (oc.is_rational()) {
                                         m.overall_coeff = *_num1_p;
-                                        ex mm = m.eval();
-                                        add_to_pomap(pomap, the_pow.op(0), mm, oc);
+                                        e = m.eval();
                                 }
                         }
+                        add_to_pomap(pomap, the_pow.op(0), e, oc);
                 }
         }
         else if (is_exactly_a<add>(*this)) {
@@ -138,23 +164,6 @@ void ex::collect_powers(power_ocvector_map& pomap) const
         }
         else if (is_exactly_a<function>(*this)) {
                 const function& f = ex_to<function>(*this);
-                if (f.get_serial() == exp_SERIAL::serial) {
-                        if (is_exactly_a<numeric>(f.op(0))) {
-                                numeric n = ex_to<numeric>(f.op(0));
-                                if (n.is_rational())
-                                        add_to_pomap(pomap, symbol_E, _ex1, n);
-                        }
-                        if (is_exactly_a<mul>(f.op(0))) {
-                                mul m = ex_to<mul>(f.op(0));
-                                numeric oc = m.overall_coeff;
-                                if (oc.is_rational()) {
-                                        m.overall_coeff = *_num1_p;
-                                        ex mm = m.eval();
-                                        add_to_pomap(pomap, symbol_E, mm, oc);
-                                }
-                        }
-                }
-                else
                         add_to_pomap(pomap, f, _ex1, *_num1_p);
         }
         else if (is_exactly_a<constant>(*this)
@@ -170,12 +179,15 @@ static void transform_powers(power_ocvector_map& pomap)
                 for (const numeric& num : it.second) {
                         g = g.gcd(num);
                 }
-                (it.second)[0] = g;
+                if (g.is_integer())
+                        (it.second)[0] = *_num1_p;
+                else
+                        (it.second)[0] = g;
         }
 }
 
 // Convert to Singular polynomial over QQ, filling replacement dicts
-const CanonicalForm ex::to_canonical(ex_int_map& map,
+const CanonicalForm ex::to_canonical(ex_int_map& amap,
                 power_ocvector_map& pomap,
                 exvector& revmap) const
 {
@@ -184,48 +196,23 @@ const CanonicalForm ex::to_canonical(ex_int_map& map,
                 const add& a = ex_to<add>(*this);
                 CanonicalForm p(0);
                 for (const auto& termex : a.seq) {
-                        p = p + a.recombine_pair_to_ex(termex).to_canonical(map, pomap, revmap);
+                        p = p + a.recombine_pair_to_ex(termex).to_canonical(amap, pomap, revmap);
                 }
-                p = p + a.overall_coeff.to_canonical();
+                p = p + num2canonical(a.overall_coeff, amap, revmap);
                 return p;
         }
         else if (is_exactly_a<numeric>(*this))
         {
-                numeric num = ex_to<numeric>(*this);
-                if (num.is_real()) {
-                        if (num.is_rational())
-                                return num2canonical(num);
-                        else
-                                return replace_with_symbol(num, map, revmap);
-                } else { // complex
-                        numeric re = num.real();
-                        numeric im = num.imag();
-                        CanonicalForm re_p, im_p;
-                        if (re.is_rational())
-                                re_p = num2canonical(re);
-                        else
-                                re_p = replace_with_symbol(re, map, revmap);
-                        if (im.is_rational())
-                                im_p = num2canonical(im);
-                        else
-                                im_p = replace_with_symbol(im, map, revmap);
-                        return re_p + im_p * replace_with_symbol(I, map, revmap);
-                }
+                return num2canonical(ex_to<numeric>(*this), amap, revmap);
         }
         else if (is_exactly_a<mul>(*this))
         {
                 const mul& m = ex_to<mul>(*this);
-                CanonicalForm p = num2canonical(*_num1_p);
+                CanonicalForm p = num2canonical(*_num1_p, amap, revmap);
                 for (const auto& termex : m.seq) {
-                        p = p * m.recombine_pair_to_ex(termex).to_canonical(map, pomap, revmap);
-                }
-                CanonicalForm oc;
-                try {
-                         oc = m.overall_coeff.to_canonical();
-                }
-                catch (std::runtime_error) {
-                        oc = replace_with_symbol(m.overall_coeff, map, revmap);
+                        p = p * m.recombine_pair_to_ex(termex).to_canonical(amap, pomap, revmap);
                 }
+                CanonicalForm oc = num2canonical(m.overall_coeff, amap, revmap);
                 p = p * oc;
                 return p;
         }
@@ -238,62 +225,53 @@ const CanonicalForm ex::to_canonical(ex_int_map& map,
                                 CanonicalForm var;
                                 power_ocvector_map::iterator it;
                                 numeric n;
-                                if (expo.is_negative()) {
-                                        var = replace_with_symbol(power(pow.basis, _ex_1),
-                                                        map, revmap);
-                                        it = pomap.find(power(pow.basis,
-                                                                _ex_1));
-                                        if (it == pomap.end())
-                                                throw std::runtime_error("can't happen in ex::to_canonical");
-                                        n = -expo.div(it->second[0]);
-                                }
-                                else {
-                                        var = replace_with_symbol(pow.basis, map, revmap);
-                                        it = pomap.find(pow.basis);
-                                        if (it == pomap.end())
-                                                throw std::runtime_error("can't happen in ex::to_canonical");
-                                        n = expo.div(it->second[0]);
-                                }
-                                revmap[var.level()-1] = GiNaC::power(it->first,
+                                var = replace_with_symbol(pow.basis, amap, revmap);
+                                it = pomap.find(pow.basis);
+                                if (it == pomap.end())
+                                        throw std::runtime_error("can't happen in ex::to_canonical");
+                                n = expo.div(it->second[0]);
+                                ex b = it->first.subs(symbol_E == exp(1));
+                                revmap[var.level()-1] = GiNaC::power(b,
                                                                it->second[0]);
-                                return ::power(var, n.to_int());
+                                try {
+                                        return ::power(var, n.to_long());
+                                }
+                                catch (std::runtime_error) {
+                                        throw std::runtime_error("exponent too big");
+                                }
                         }
                 }
-                if (is_exactly_a<mul>(pow.exponent)) {
-                        mul m = ex_to<mul>(pow.exponent);
-                        numeric oc = m.overall_coeff;
-                        if (oc.is_rational()) {
-                                CanonicalForm var;
-                                power_ocvector_map::iterator it;
-                                numeric n;
-                                if (oc.is_negative()) {
-                                        m.overall_coeff = *_num_1_p;
-                                        ex mm = m.eval();
-                                        it = pomap.find(GiNaC::power(pow.basis, mm));
-                                        if (it == pomap.end())
-                                                throw std::runtime_error("can't happen in ex::to_canonical");
-                                        var = replace_with_symbol(it->first,
-                                                                  map, revmap);
-                                        n = -oc.div(it->second[0]);
-                                }
-                                else {
+                else {
+                        numeric oc = *_num1_p;
+                        ex e = pow.exponent;
+                        if (is_exactly_a<mul>(e)) {
+                                mul m = ex_to<mul>(e);
+                                oc = m.overall_coeff;
+                                if (oc.is_rational()) {
+                                        power_ocvector_map::iterator it;
                                         m.overall_coeff = *_num1_p;
-                                        ex mm = m.eval();
-                                        it = pomap.find(GiNaC::power(pow.basis, mm));
-                                        if (it == pomap.end())
-                                                throw std::runtime_error("can't happen in ex::to_canonical");
-                                        var = replace_with_symbol(it->first,
-                                                                  map, revmap);
-                                        n = oc.div(it->second[0]);
+                                        e = m.eval();
                                 }
-                                revmap[var.level()-1] = GiNaC::power(it->first, it->second[0]);
-                                return ::power(var, n.to_int());
+                        }
+                        auto it = pomap.find(GiNaC::power(pow.basis, e));
+                        if (it == pomap.end())
+                                throw std::runtime_error("can't happen in ex::to_canonical");
+                        CanonicalForm var = replace_with_symbol(it->first,
+                                                            amap, revmap);
+                        numeric n = oc.div(it->second[0]);
+                        ex b = it->first.subs(symbol_E == exp(1));
+                        revmap[var.level()-1] = GiNaC::power(b, it->second[0]);
+                        try {
+                                return ::power(var, n.to_long());
+                        }
+                        catch (std::runtime_error) {
+                                throw std::runtime_error("exponent too big");
                         }
                 }
-                return replace_with_symbol(*this, map, revmap);
+                return replace_with_symbol(*this, amap, revmap);
         }
 
-        return replace_with_symbol(*this, map, revmap);
+        return replace_with_symbol(*this, amap, revmap);
 }
 
 static numeric can2num(const CanonicalForm& f)
@@ -465,7 +443,7 @@ factored_b:
 			} else {
 				ex p_co, pb_co;
 				ex p_gcd = gcdpoly(p, pb, &p_co, &pb_co, check_args);
-				if (p_gcd.is_equal(_ex1)) {
+				if (p_gcd.is_one()) {
 					// a(x) = p(x)^n, b(x) = p_b(x)^m, gcd (p, p_b) = 1 ==>
 					// gcd(a,b) = 1
 					if (ca != nullptr)
@@ -501,7 +479,7 @@ factored_b:
 			ex p_co, bpart_co;
 			ex p_gcd = gcdpoly(p, b, &p_co, &bpart_co, false);
 
-			if (p_gcd.is_equal(_ex1)) {
+			if (p_gcd.is_one()) {
 				// a(x) = p(x)^n, gcd(p, b) = 1 ==> gcd(a, b) = 1
 				if (ca != nullptr)
 					*ca = a;
@@ -528,7 +506,7 @@ factored_b:
 		ex p_co, apart_co;
 		const ex& exp_b(b.op(1));
 		ex p_gcd = gcdpoly(a, p, &apart_co, &p_co, false);
-		if (p_gcd.is_equal(_ex1)) {
+		if (p_gcd.is_one()) {
 			// b=p(x)^n, gcd(a, p) = 1 ==> gcd(a, b) == 1
 			if (ca != nullptr)
 				*ca = a;
@@ -546,17 +524,24 @@ factored_b:
 
         ex_int_map map;
         exvector revmap;
+        map.insert(std::make_pair(symbol_E, 1));
+        revmap.push_back(exp(1));
         On(SW_RATIONAL);
         setCharacteristic(0);
         power_ocvector_map pomap;
-        a.collect_powers(pomap);
-        b.collect_powers(pomap);
+        ex aa = a.subs(exp(wild()) == pow(symbol_E, wild())).expand();
+        ex bb = b.subs(exp(wild()) == pow(symbol_E, wild())).expand();
+        aa.collect_powers(pomap);
+        bb.collect_powers(pomap);
+//        Log(pomap,"pomap");
         transform_powers(pomap);
-        CanonicalForm p = a.to_canonical(map, pomap, revmap);
-        CanonicalForm q = b.to_canonical(map, pomap, revmap);
+//        Log(map,"map");
+//        Log(revmap,"revmap");
+//        Log(pomap,"pomap after transform");
+        CanonicalForm p = aa.to_canonical(map, pomap, revmap);
+        CanonicalForm q = bb.to_canonical(map, pomap, revmap);
         CanonicalForm d = gcd(p, q);
         ex res = canonical_to_ex(d, revmap);
-
         if (ca != nullptr) {
                 ex quo;
                 if (divide(a, res, quo))
@@ -617,11 +602,19 @@ bool factorpoly(const ex& the_ex, ex& res_prod)
 
         ex_int_map map;
         exvector revmap;
+        map.insert(std::make_pair(symbol_E, 1));
+        revmap.push_back(exp(1));
 
+        On(SW_RATIONAL);
         power_ocvector_map pomap;
-        the_ex.collect_powers(pomap);
+        ex e = the_ex.subs(exp(wild()) == pow(symbol_E, wild()));
+        e.collect_powers(pomap);
+        //Log(pomap,"pomap");
         transform_powers(pomap);
-        CanonicalForm p = the_ex.to_canonical(map, pomap, revmap);
+        //Log(map,"map");
+        //Log(revmap,"revmap");
+        //Log(pomap,"pomap after transform");
+        CanonicalForm p = e.to_canonical(map, pomap, revmap);
         CFFList factors = factorize(p);
 
         if (factors.length() == 1 or factors.isEmpty())
@@ -662,6 +655,8 @@ ex resultantpoly(const ex & ee1, const ex & ee2, const ex & s)
 {
         ex_int_map map;
         exvector revmap;
+        map.insert(std::make_pair(symbol_E, 1));
+        revmap.push_back(exp(1));
         On(SW_RATIONAL);
         setCharacteristic(0);
         power_ocvector_map pomap;
