@@ -156,9 +156,7 @@ void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
 		//(coeffstr.find('/') != std::string::npos) ||
 		//(coeffstr.find('*') != std::string::npos) ||
 		//(coeffstr.find('^') != std::string::npos));
-	if (num_coeff.is_integer()
-                and num_coeff.is_minus_one()
-                and not num_coeff.is_parent_pos_char())
+	if (num_coeff.is_minus_one())
 		c.s<<"-";
 	else if (parenthesis && coeffstr[0] == '-') {
 		// We want to move the '-' out of the parenthesis if it is
@@ -278,9 +276,7 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 		if (latex_tags) {
 			const numeric& numer = overall_coeff.numer();
 			bool negate = false;
-			if (numer.is_integer()
-                            and numer.is_minus_one()
-                            and not numer.is_parent_pos_char()) {
+			if (numer.is_minus_one()) {
 				c.s<<"-";
 				negate = true;
 				//print_numer = coeff_numer.mul(*_num_1_p);
@@ -313,8 +309,8 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 				}
 			}
 			c.s << "}{";
-			ex denom = overall_coeff.denom();
-			if (denom.is_equal(_ex1)) {
+			numeric denom = overall_coeff.denom();
+			if (denom.is_one()) {
 				mul(neg_powers).eval().print(c);
 			} else {
 				mul(denom, mul(neg_powers).eval()).hold().print(c);
@@ -398,41 +394,39 @@ void mul::do_print_csrc(const print_csrc & c, unsigned level) const
 	}
 
 	// Print arguments, separated by "*" or "/"
-	auto it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-
+        bool seqstart = true;
+	for (const auto & item : seq) {
+                const numeric co = ex_to<numeric>(item.coeff);
 		// If the first argument is a negative integer power, it gets printed as "1.0/<expr>"
 		bool needclosingparenthesis = false;
-		if (it == seq.begin() && it->coeff.info(info_flags::negint)) {
+		if (seqstart and co.info(info_flags::negint)) {
 			if (is_a<print_csrc_cl_N>(c)) {
 				c.s << "recip(";
 				needclosingparenthesis = true;
 			} else
 				c.s << "1.0/";
+                        seqstart = false;
 		}
 
 		// If the exponent is 1 or -1, it is left out
-		if (it->coeff.is_integer_pmone()) {
-			it->rest.print(c, precedence());
+		if (co.is_one() or co.is_minus_one()) {
+			item.rest.print(c, precedence());
 		}
-		else if (it->coeff.info(info_flags::negint))
+		else if (co.info(info_flags::negint))
 			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, -ex_to<numeric>(it->coeff)))).print(c, level);
+			(ex(power(item.rest, -co))).print(c, level);
 		else
 			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, ex_to<numeric>(it->coeff)))).print(c, level);
+			(ex(power(item.rest, co))).print(c, level);
 
 		if (needclosingparenthesis)
 			c.s << ")";
 
 		// Separator is "/" for negative integer powers, "*" otherwise
-		++it;
-		if (it != itend) {
-			if (it->coeff.info(info_flags::negint))
-				c.s << "/";
-			else
-				c.s << "*";
-		}
+                if (co.info(info_flags::negint))
+                        c.s << "/";
+                else
+                        c.s << "*";
 	}
 
 	if (precedence() <= level)
@@ -571,12 +565,12 @@ bool mul::is_polynomial(const ex & var) const
 	return true;
 }
 
-int mul::degree(const ex & s) const
+numeric mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
-	int deg_sum = 0;
+	numeric deg_sum(0);
         for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_integer())
+		if (ex_to<numeric>(elem.coeff).is_real())
 			deg_sum += recombine_pair_to_ex(elem).degree(s);
 		else {
 			if (elem.rest.has(s))
@@ -585,12 +579,12 @@ int mul::degree(const ex & s) const
 	return deg_sum;
 }
 
-int mul::ldegree(const ex & s) const
+numeric mul::ldegree(const ex & s) const
 {
 	// Sum up degrees of factors
-	int deg_sum = 0;
+	numeric deg_sum(0);
         for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_integer())
+		if (ex_to<numeric>(elem.coeff).is_real())
 			deg_sum += recombine_pair_to_ex(elem).ldegree(s);
 		else {
 			if (elem.rest.has(s))
@@ -663,7 +657,7 @@ ex mul::eval(int level) const
 	
 	if (level == 1 and is_evaluated()) {
 		GINAC_ASSERT(seq.size()>0);
-		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_integer_one());
+		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_one());
 		return *this;
 	}
 
@@ -674,12 +668,21 @@ ex mul::eval(int level) const
 		           setflag(status_flags::dynallocated);
 	}
 	
-	// handle infinity and handle exp(a)*exp(b) -> exp(a+b) and
+	// handle infinity, simple way
+        if (std::any_of(seq.cbegin(), seq.cend(),
+                        [](expair p)
+                                { return is_exactly_a<infinity>(p.rest); })) {
+                infinity result = infinity::from_sign(1);
+                result *= overall_coeff;
+                for (auto p : seq)
+		        result *= recombine_pair_to_ex(p);
+                return result;
+        }
+
+        // handle exp(a)*exp(b) -> exp(a+b) and
 	unsigned exp_count = 0;
 	for (auto i = seq.begin(); i != seq.end(); i++) {
 		const numeric& num_coeff = ex_to<numeric>(i->coeff);
-		if (unlikely(is_exactly_a<infinity>(i->rest)))
-			return eval_infinity(i);
 		if (unlikely(is_ex_the_function(i->rest, exp) and
 			     num_coeff.is_integer())) {
 			exp_count++;
@@ -698,8 +701,7 @@ ex mul::eval(int level) const
 		// *(;c) -> c
 		return overall_coeff;
 	} else if (seq_size==1
-                   and overall_coeff.is_one()
-		   and not overall_coeff.is_parent_pos_char()) {
+                   and overall_coeff.is_one()) {
 		// *(x;1) -> x
 		// except in positive characteristic: 1*(x+2) = x in F_2
 		return recombine_pair_to_ex(*(seq.begin()));
@@ -731,7 +733,7 @@ ex mul::eval(int level) const
 		numeric oc = *_num1_p;
 		bool something_changed = false;
 		while (i!=last) {
-			if (likely(! (is_exactly_a<add>(i->rest) && i->coeff.is_integer_one()))) {
+			if (likely(! (is_exactly_a<add>(i->rest) && i->coeff.is_one()))) {
 				// power::eval has such a rule, no need to handle powers here
 				++i;
 				continue;
@@ -843,23 +845,6 @@ ex mul::eval_exponentials() const
 	return result->setflag(status_flags::dynallocated);
 }
 
-
-ex mul::eval_infinity(epvector::const_iterator infinity_iter) const
-{
-	GINAC_ASSERT(is_exactly_a<infinity>(infinity_iter->rest));
-	GINAC_ASSERT(infinity_iter->coeff.is_equal(_ex1));
-	infinity result(ex_to<numeric>(
-                ex_to<infinity>(infinity_iter->rest).get_direction()));
-        result *= overall_coeff;
-
-	for (auto i = seq.begin(); i != seq.end(); i++) {
-		if (i == infinity_iter) continue;
-		result *= recombine_pair_to_ex(*i);
-	}
-	return result;
-}
-
-
 ex mul::evalf(int level, PyObject* parent) const
 {
 	if (level==1)
@@ -1237,7 +1222,7 @@ expair mul::combine_pair_with_coeff_to_pair(const expair & p,
                 return expair(p.rest, p.coeff*c);
 	if (c.is_one())
 		return p;
-        if (p.coeff.is_integer_one())
+        if (p.coeff.is_one())
                 return expair(p.rest, c);
 
 	// to avoid duplication of power simplification rules,
@@ -1249,9 +1234,12 @@ expair mul::combine_pair_with_coeff_to_pair(const expair & p,
 	
 ex mul::recombine_pair_to_ex(const expair & p) const
 {
-        if (unlikely(is_exactly_a<infinity>(p.rest)))
-                return (new infinity(ex_to<numeric>(p.coeff)))->setflag(status_flags::evaluated|status_flags::dynallocated);
-	if (p.coeff.is_integer_one()) 
+        if (unlikely(is_exactly_a<infinity>(p.rest))) {
+                infinity res = ex_to<infinity>(p.rest);
+                res *= p.coeff;
+                return res;
+        }
+	if (p.coeff.is_one()) 
 		return p.rest;
 	else
 		return (new power(p.rest,p.coeff))->setflag(status_flags::dynallocated);
@@ -1272,7 +1260,7 @@ bool mul::expair_needs_further_processing(epp it)
 			*it = ep;
 			return true;
 		}
-		if (it->coeff.is_integer_one()) {
+		if (it->coeff.is_one()) {
 			// combined pair has coeff 1 and must be moved to the end
 			return true;
 		}
@@ -1349,7 +1337,7 @@ ex mul::expand(unsigned options) const
 
 	for (const auto & elem : expanded_seq) {
 		if (is_exactly_a<add>(elem.rest) &&
-			(elem.coeff.is_integer_one())) {
+			(elem.coeff.is_one())) {
 			if (is_exactly_a<add>(last_expanded)) {
 				// Expand a product of two sums, aggressive version.
 				// Caring for the overall coefficients in separate loops can
@@ -1425,7 +1413,7 @@ ex mul::expand(unsigned options) const
 				} 
 				last_expanded = tmp_accu;
 			} else {
-				if (!last_expanded.is_integer_one())
+				if (!last_expanded.is_one())
 					non_adds.push_back(split_ex_to_pair(last_expanded));
 				last_expanded = elem.rest;
 			}
