@@ -119,7 +119,7 @@ void power::print_power(const print_context & c, const char *powersymbol, const
 
 void power::do_print_dflt(const print_dflt & c, unsigned level) const
 {
-       if ((-exponent).is_integer_one()) {
+       if (exponent.is_minus_one()) {
 		// inverses printed in a special way
  	        if (level >= 20) c.s << "(";
 		c.s << "1/";
@@ -152,13 +152,13 @@ void power::do_print_dflt(const print_dflt & c, unsigned level) const
 		// exp function prints as e^a. Printing powers of this can be
 		// confusing, so we add parenthesis if the basis is exp
 		bool exp_parenthesis = is_ex_the_function(basis, exp) and
-                        not basis.op(0).is_integer_one();
+                        not basis.op(0).is_one();
 		if (exp_parenthesis)
 			c.s << '(';
 		basis.print(c, precedence());
 		if (exp_parenthesis)
 			c.s << ')';
-		if (not (-exponent).is_integer_one()) {
+		if (not exponent.is_minus_one()) {
     		        c.s << "^" << expstr;
 		}
 		if (precedence() <= level)
@@ -185,7 +185,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 		// exp function prints as e^a. Printing powers of this can be
 		// confusing, so we add parenthesis if the basis is exp
 		bool base_parenthesis = is_ex_the_function(basis, exp) and
-			not basis.op(0).is_integer_one();
+			not basis.op(0).is_one();
 
 		if (precedence() <= level)
 			c.s << "{\\left(";
@@ -197,7 +197,7 @@ void power::do_print_latex(const print_latex & c, unsigned level) const
 		if (base_parenthesis)
 			c.s << "\\right)";
 
-		if (not (-exponent).is_integer_one()) {
+		if (not exponent.is_minus_one()) {
     		        c.s << "^{";
 			bool exp_parenthesis = is_exactly_a<power>(exponent);
 			if (exp_parenthesis)
@@ -240,7 +240,7 @@ static void print_sym_pow(const print_context & c, const symbol &x, int exp)
 void power::do_print_csrc(const print_csrc & c, unsigned level) const
 {
 	if (is_a<print_csrc_cl_N>(c)) {
-		if (exponent.is_integer_one()) {
+		if (exponent.is_one()) {
 			c.s << "recip(";
 			basis.print(c);
 			c.s << ')';
@@ -269,7 +269,7 @@ void power::do_print_csrc(const print_csrc & c, unsigned level) const
 		c.s << ')';
 
 	// <expr>^-1 is printed as "1.0/<expr>" or with the recip() function of CLN
-	} else if (exponent.is_integer_one()) {
+	} else if (exponent.is_one()) {
 		c.s << "1.0/(";
 		basis.print(c);
 		c.s << ')';
@@ -384,30 +384,30 @@ bool power::is_polynomial(const ex & var) const
 	return false;
 }
 
-int power::degree(const ex & s) const
+numeric power::degree(const ex & s) const
 {
 	if (is_equal(ex_to<basic>(s)))
 		return 1;
-	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
+	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_real()) {
 		if (basis.is_equal(s))
-			return ex_to<numeric>(exponent).to_int();
+			return ex_to<numeric>(exponent);
 		else
-			return basis.degree(s) * ex_to<numeric>(exponent).to_int();
+			return basis.degree(s) * ex_to<numeric>(exponent);
 	} else if (basis.has(s))
 		throw(std::runtime_error("power::degree(): undefined degree because of non-integer exponent"));
 	else
 		return 0;
 }
 
-int power::ldegree(const ex & s) const 
+numeric power::ldegree(const ex & s) const 
 {
 	if (is_equal(ex_to<basic>(s)))
 		return 1;
-	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
+	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_real()) {
 		if (basis.is_equal(s))
-			return ex_to<numeric>(exponent).to_int();
+			return ex_to<numeric>(exponent);
 		else
-			return basis.ldegree(s) * ex_to<numeric>(exponent).to_int();
+			return basis.ldegree(s) * ex_to<numeric>(exponent);
 	} else if (basis.has(s))
 		throw(std::runtime_error("power::ldegree(): undefined degree because of non-integer exponent"));
 	else
@@ -417,7 +417,7 @@ int power::ldegree(const ex & s) const
 ex power::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
-		return n.is_integer_one() ? _ex1 : _ex0;
+		return n.is_one() ? _ex1 : _ex0;
 	else if (!basis.is_equal(s)) {
 		// basis not equal to s
 		if (n.is_zero())
@@ -522,14 +522,11 @@ ex power::eval(int level) const
 	}
 	
 	// ^(x,0) -> 1  (0^0 also handled here)
-	if (eexponent.is_zero() && 
-		!(basis_is_numerical && 
-			num_basis.is_parent_pos_char())) {
+	if (eexponent.is_zero())
 		return _ex1;
-	}
 	
 	// ^(x,1) -> x
-	if (eexponent.is_integer_one())
+	if (eexponent.is_one())
 		return ebasis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
@@ -543,20 +540,30 @@ ex power::eval(int level) const
 	}
 
 	// ^(1,x) -> 1
-	if (ebasis.is_integer_one())
+	if (ebasis.is_one())
 		return _ex1;
 
 	// power of a function calculated by separate rules defined for this function
-	if (is_exactly_a<function>(ebasis))
+        if (is_exactly_a<function>(ebasis)) {
+                const function& f = ex_to<function>(ebasis);
+                if (f.get_serial() == exp_SERIAL::serial) {
+                        if (is_exactly_a<numeric>(f.op(0))
+                            or exponent_is_numerical)
+                                return exp(mul(eexponent, f.op(0)));
+                }
 		return ex_to<function>(ebasis).power(eexponent);
+        }
 
 	// Turn (x^c)^d into x^(c*d) in the case that x is positive and c is real,
         // or if d is integer (and positive to preserve fraction output).
-	if (is_exactly_a<power>(ebasis) and ((eexponent.info(info_flags::integer)
-                                          and eexponent.info(info_flags::positive))
-                                        or (ebasis.op(0).info(info_flags::positive)
-                                          and ebasis.op(1).info(info_flags::real))))
-		return power(ebasis.op(0), ebasis.op(1) * eexponent);
+        // Secondly, if both c,d are numeric negative then cancel minuses.
+	if (is_exactly_a<power>(ebasis)) {
+                if (((eexponent.info(info_flags::integer)
+                      and eexponent.info(info_flags::positive))
+                    or (ebasis.op(0).info(info_flags::positive)
+                      and ebasis.op(1).info(info_flags::real))))
+		        return power(ebasis.op(0), ebasis.op(1) * eexponent);
+        }
 
         // (negative oc)^even --> (positive oc)^even
         if (is_exactly_a<mul>(ebasis)
@@ -641,6 +648,11 @@ ex power::eval(int level) const
 			const ex & sub_exponent = sub_power.exponent;
 			if (is_exactly_a<numeric>(sub_exponent)) {
 				const numeric & num_sub_exponent = ex_to<numeric>(sub_exponent);
+                                if (num_sub_exponent.is_negative()
+                                    and num_exponent.is_negative())
+                                        return power(power(sub_basis,
+                                                           -num_sub_exponent),
+                                                     -num_exponent);
 				GINAC_ASSERT(num_sub_exponent!=numeric(1));
 				if (num_exponent.is_integer() || (abs(num_sub_exponent) - (*_num1_p)).is_negative() 
 						|| (num_sub_exponent == *_num_1_p && num_exponent.is_positive())) {
@@ -669,7 +681,7 @@ ex power::eval(int level) const
 
                                 const bool canonicalizable = lead_coeff.is_integer();
                                 const add& addref = ex_to<add>(ebasis);
-                                const bool unit_normal = lead_coeff.is_positive() || lead_coeff.is_parent_pos_char();
+                                const bool unit_normal = lead_coeff.is_positive();
                                 if (canonicalizable
                                     and (! unit_normal)
                                     and num_exponent.denom().is_one())
@@ -744,12 +756,15 @@ ex power::eval(int level) const
                                                 outex = outer[0];
                                         else
                                                 outex = mul(outer).hold();
-                                        if (inner.size() == 1)
-                                                innex = inner[0];
-                                        else
-                                                innex = mul(inner).hold();
-                                        ex p = power(innex, eexponent).hold();
-                                        return (new mul(outex, p))->setflag(status_flags::dynallocated | status_flags::evaluated);
+                                        if (inner.size() > 0) {
+                                                if (inner.size() == 1)
+                                                        innex = inner[0];
+                                                else
+                                                        innex = mul(inner).hold();
+                                                ex p = power(innex, eexponent).hold();
+                                                return (new mul(outex, p))->setflag(status_flags::dynallocated | status_flags::evaluated);
+                                        }
+                                        return outex;
                                 }
                         }
                 }
@@ -1396,7 +1411,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 			const std::vector<int>& partition = partitions.current();
 			// All monomials of this partition have the same number of terms and the same coefficient.
 			const unsigned msize = std::count_if(partition.begin(), partition.end(), [](int i) { return i > 0; });
-			const numeric coeff = multinomial_coefficient(partition) * binomial_coefficient;
+			const numeric coef = multinomial_coefficient(partition) * binomial_coefficient;
 
 			// Iterate over all compositions of the current partition.
 			composition_generator compositions(partition);
@@ -1404,7 +1419,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 				const std::vector<int>& the_exponent = compositions.current();
 				epvector monomial;
 				monomial.reserve(msize);
-				numeric factor = coeff;
+				numeric factor = coef;
 				for (unsigned i = 0; i < the_exponent.size(); ++i) {
 					const ex & r = a.seq[i].rest;
 					GINAC_ASSERT(!is_exactly_a<add>(r));
@@ -1473,7 +1488,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		             !is_exactly_a<mul>(ex_to<power>(r).basis) ||
 		             !is_exactly_a<power>(ex_to<power>(r).basis));
 		
-		if (c.is_integer_one()) {
+		if (c.is_one()) {
 			if (is_exactly_a<mul>(r)) {
 				result.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
 				                        _ex1));
